from abc import ABC, abstractmethod
from datetime import datetime
from typing import Union

from cloud_governance.main.environment_variables import environment_variables


class AbstractCleanUpOperations(ABC):

    DAYS_TO_NOTIFY_ADMINS = 2
    DAYS_TO_TRIGGER_RESOURCE_MAIL = 4
    DAILY_HOURS = 24
    CURRENT_DATE = datetime.utcnow().date().__str__()

    def __init__(self):
        self._environment_variables_dict = environment_variables.environment_variables_dict
        self._days_to_take_action = self._environment_variables_dict.get('DAYS_TO_TAKE_ACTION')
        self._dry_run = self._environment_variables_dict.get('dry_run')
        self._policy = self._environment_variables_dict.get('policy')
        self._force_delete = self._environment_variables_dict.get('FORCE_DELETE')
        self._resource_id = self._environment_variables_dict.get('RESOURCE_ID')

    @abstractmethod
    def get_clean_up_days_count(self, tags: Union[list, dict]):
        """
        This method returns the cleanup days count
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    @abstractmethod
    def get_tag_name_from_tags(self, tags: Union[list, dict], tag_name: str):
        """
        This method returns the tag_value from the tags
        :param tags:
        :type tags:
        :param tag_name:
        :type tag_name:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    def get_skip_policy_value(self, tags: Union[list, dict]) -> str:
        """
        This method returns the skip value
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        policy_value = self.get_tag_name_from_tags(tags=tags, tag_name='Policy').strip()
        if not policy_value:
            policy_value = self.get_tag_name_from_tags(tags=tags, tag_name='Skip').strip()
        if policy_value:
            return policy_value.replace('_', '').replace('-', '').upper()
        return 'NA'

    @abstractmethod
    def _delete_resource(self, resource_id: str):
        """
        This method deletes the resource
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    @abstractmethod
    def update_resource_day_count_tag(self, resource_id: str, cleanup_days: int, tags: list):
        """
        This method updates the resource tags
        :param resource_id:
        :type resource_id:
        :param cleanup_days:
        :type cleanup_days:
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    def verify_and_delete_resource(self, resource_id: str, tags: list, clean_up_days: int,
                                   days_to_delete_resource: int = None, **kwargs):
        """
        This method verify and delete the resource by calculating the days
        :return:
        :rtype:
        """
        if self._resource_id == resource_id and self._force_delete and self._dry_run == 'no':
            self._delete_resource(resource_id=resource_id)
            return True
        if not days_to_delete_resource:
            days_to_delete_resource = self._days_to_take_action
        cleanup_resources = False
        if clean_up_days >= self._days_to_take_action - self.DAYS_TO_TRIGGER_RESOURCE_MAIL:
            if clean_up_days == self._days_to_take_action - self.DAYS_TO_TRIGGER_RESOURCE_MAIL:
                kwargs['delta_cost'] = kwargs.get('extra_purse')
                # @Todo, If it require add email alert. May In future will add the email alert.
            else:
                if clean_up_days >= days_to_delete_resource:
                    if self._dry_run == 'no':
                        if self.get_skip_policy_value(tags=tags) not in ('NOTDELETE', 'SKIP'):
                            self._delete_resource(resource_id=resource_id)
                            cleanup_resources = True
        return cleanup_resources



import os

from cloud_governance.main.environment_variables import environment_variables
from cloud_governance.policy.policy_runners.aws.policy_runner import PolicyRunner


class AWSMainOperations:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__policy = self.__environment_variables_dict.get('policy', '')
        self.__policy_runner = PolicyRunner()

    def __get_policies(self) -> dict:
        """
        This method gets the aws policies
        :return:
        :rtype:
        """
        policies = {}
        policies_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'policy', 'aws')
        for (dirpath, dirnames, filenames) in os.walk(policies_path):
            immediate_parent = dirpath.split("/")[-1]
            for filename in filenames:
                if not filename.startswith('__') and (filename.endswith('.yml') or filename.endswith('.py')):
                    policies.setdefault(immediate_parent, []).append(os.path.splitext(filename)[0])
        return policies

    def run(self):
        """
        This method run the AWS Policy operations
        :return:
        :rtype:
        """
        policies_list = self.__get_policies()
        for policy_type, policies in policies_list.items():
            # @Todo support for all the aws policies, currently supports ec2_run as urgent requirement
            if self.__policy in policies and self.__policy == "ec2_run":
                self.__policy_runner.run(source=policy_type)
                return True
        return False



import datetime

from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.run_zombie_non_cluster_policies import \
    NonClusterZombiePolicy


class EC2Run(NonClusterZombiePolicy):

    RESOURCE_ACTION = "Stopped"

    def __init__(self):
        super(EC2Run, self).__init__()
        self.__es_index = 'cloud-governance-ec2-instance-types'

    def __update_instance_type_count(self, instances: list):
        """
        This method updates the instance types count
        :param instances:
        :type instances:
        :return:
        :rtype:
        """
        instance_types = {}
        for instance in instances:
            instance_type = instance.get('InstanceType')
            instance_types[instance_type] = instance_types.get(instance_type, 0) + 1
        es_instance_types_data = []
        for key, value in instance_types.items():
            es_instance_types_data.append({
                'instance_type': key,
                'instance_count': value,
                'timestamp': datetime.datetime.utcnow(),
                'region': self._region,
                'account': self._account.upper().replace('OPENSHIFT-', ''),
                'index_id': f'{key}-{self._account.lower()}-{self._region}-{str(datetime.datetime.utcnow().date())}'
            })
        self._es_upload.es_upload_data(items=es_instance_types_data, es_index=self.__es_index, set_index='index_id')

    def __ec2_run(self):
        """
        This method list the running instances and upload to elastic_search
        :return:
        :rtype:
        """
        instances = self._ec2_operations.get_ec2_instance_list()
        self.__update_instance_type_count(instances=instances)
        running_instances_data = []
        for instance in instances:
            tags = instance.get('Tags', [])
            if instance.get('State', {}).get('Name') == 'running':
                running_days = self._calculate_days(instance.get('LaunchTime'))
                cleanup_days = self._aws_cleanup_policies.get_clean_up_days_count(tags=tags)
                cleanup_result = self._aws_cleanup_policies.verify_and_delete_resource(
                    resource_id=instance.get('InstanceId'), tags=tags,
                    clean_up_days=cleanup_days)
                resource_data = {
                        'ResourceId': instance.get('InstanceId'),
                        'User': self._get_tag_name_from_tags(tags=tags, tag_name='User'),
                        'SkipPolicy': self._aws_cleanup_policies.get_skip_policy_value(tags=tags),
                        'LaunchTime': instance['LaunchTime'].strftime("%Y-%m-%dT%H:%M:%S+00:00"),
                        'InstanceType': instance.get('InstanceType'),
                        'InstanceState': instance.get('State', {}).get('Name') if not cleanup_result else 'stopped',
                        'StateTransitionReason': instance.get('StateTransitionReason'),
                        'RunningDays': running_days,
                        'CleanUpDays': cleanup_days,
                        'DryRun': self._dry_run,
                        'Name': self._get_tag_name_from_tags(tags=tags, tag_name='Name'),
                        'RegionName': self._region,
                        f'Resource{self.RESOURCE_ACTION}': str(cleanup_result)
                }
                if self._force_delete and self._dry_run == 'no':
                    resource_data.update({'ForceDeleted': str(self._force_delete)})
                running_instances_data.append(resource_data)
            else:
                cleanup_days = 0
            self._aws_cleanup_policies.update_resource_day_count_tag(resource_id=instance.get('InstanceId'),
                                                                     cleanup_days=cleanup_days, tags=tags)

        return running_instances_data

    def run(self):
        """
        This method list all the running instances
        @return:
        """
        return self.__ec2_run()




import importlib
import inspect

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_runners.aws.upload_s3 import UploadS3
from cloud_governance.policy.policy_runners.common.abstract_policy_runner import AbstractPolicyRunner


class PolicyRunner(AbstractPolicyRunner):

    def __init__(self):
        super().__init__()

    def run(self, source: str = "", upload: bool = True):
        """
        This method run the AWS policies classes
        :param upload:
        :type upload:
        :param source:
        :type source:
        :return:
        :rtype:
        """
        source_policy = f"{source}.{self._policy}" if source else self._policy
        logger.info(f'account={self._account}, policy={self._policy}, dry_run={self._dry_run}')
        zombie_non_cluster_policy_module = importlib.import_module(f'cloud_governance.policy.aws.{source_policy}')

        policy_result = []
        ec2_operations = EC2Operations()
        upload_to_s3 = UploadS3()
        for cls in inspect.getmembers(zombie_non_cluster_policy_module, inspect.isclass):
            if self._policy.replace('_', '').replace('-', '') == cls[0].lower():
                active_regions = [self._region]
                if self._run_active_regions:
                    active_regions = ec2_operations.get_active_regions()
                    logger.info("Running the policy in All AWS active regions")
                for active_region in active_regions:
                    logger.info(f"Running the {self._policy} in Region: {active_region}")
                    self._environment_variables_dict['AWS_DEFAULT_REGION'] = active_region
                    response = cls[1]().run()
                    if isinstance(response, str):
                        logger.info(f'key: {cls[0]}, Response: {response}')
                    else:
                        policy_result.extend(response)
                        logger.info(f'key: {cls[0]}, count: {len(response)}, {response}')
                        if upload:
                            self._upload_elastic_search.upload(data=response)
                            upload_to_s3.upload(data=response)
        if self._save_to_file_path:
            self.write_to_file(data=policy_result)
